@@ -1,14 +1,15 @@
 import random
-from typing import Dict, Optional
+from typing import Dict
+from typing import Optional
 
 from negmas import MechanismState
 from negmas import ResponseType
+from negmas.outcomes import Outcome
 from negmas.sao import SAOResponse
 from negmas.sao import SAOState
-from negmas.sao import SAOSingleAgreementRandomController, SAOSingleAgreementController
-from negmas.outcomes import Outcome
 
-from scml.oneshot.agent import OneShotAgent, OneShotSingleAgreementAgent
+from scml.oneshot.agent import OneShotAgent
+from scml.oneshot.agent import OneShotSingleAgreementAgent
 from scml.oneshot.agent import OneShotSyncAgent
 
 __all__ = ["RandomOneShotAgent", "SyncRandomOneShotAgent", "SingleAgreementRandomAgent"]
@@ -19,7 +20,7 @@ PROB_END = 0.005
 
 class RandomOneShotAgent(OneShotAgent):
     def _random_offer(self, negotiator_id: str):
-        ami = self.get_ami(negotiator_id)
+        ami = self.get_nmi(negotiator_id)
         if not ami:
             return None
         return ami.random_outcomes(1)[0]
@@ -37,7 +38,7 @@ class RandomOneShotAgent(OneShotAgent):
 
 class SyncRandomOneShotAgent(OneShotSyncAgent):
     def _random_offer(self, negotiator_id: str):
-        ami = self.get_ami(negotiator_id)
+        ami = self.get_nmi(negotiator_id)
         if not ami:
             return None
         return ami.random_outcomes(1)[0]
