@@ -1,22 +1,34 @@
-from typing import Union, Optional, Tuple, List, Any, Dict
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+from typing import Any
+from typing import Dict
+from typing import List
+from typing import Optional
+from typing import Tuple
+from typing import Union
+
 import numpy as np
 from negmas import Negotiator
-from negmas.sao import SAOController, SAONegotiator
-from ..scml2020.common import (
-    FactoryState,
-    FactoryProfile,
-    ANY_LINE,
-    ANY_STEP,
-    INFINITE_COST,
-    NO_COMMAND,
-    is_system_agent,
-)
+from negmas.sao import SAOController
+from negmas.sao import SAONegotiator
+
+from ..scml2020.common import ANY_LINE
+from ..scml2020.common import ANY_STEP
+from ..scml2020.common import INFINITE_COST
+from ..scml2020.common import NO_COMMAND
+from ..scml2020.common import FactoryProfile
+from ..scml2020.common import FactoryState
+from ..scml2020.common import is_system_agent
+
+if TYPE_CHECKING:
+    from .adapter import OneShotSCML2020Adapter
 
 
 class AWIHelper:
     """The Agent SCML2020World Interface for SCML2020 world allowing a single process per agent"""
 
-    def __init__(self, owner: "OneShotSCML2020Adapter"):
+    def __init__(self, owner: OneShotSCML2020Adapter):
         self._owner = owner
         self._world = owner.awi._world
 
