@@ -7,17 +7,15 @@ from typing import List
 from typing import Optional
 
 from negmas import Adapter
-from negmas import AgentMechanismInterface
 from negmas import Breach
 from negmas import Contract
 from negmas import Issue
 from negmas import MechanismState
 from negmas import Negotiator
-from negmas.sao import PassThroughSAONegotiator
+from negmas import NegotiatorMechanismInterface
 from negmas import RenegotiationRequest
+from negmas.sao import ControlledSAONegotiator
 
-from .common import QUANTITY, UNIT_PRICE
-from .ufun import OneShotUFun
 from .agent import OneShotAgent
 from .helper import AWIHelper
 from .mixins import OneShotUFunCreatorMixin
@@ -85,14 +83,16 @@ class DefaultOneShotAdapter(Adapter, OneShotUFunCreatorMixin):
         partners: List[str],
         issues: List[Issue],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
         role: Optional[str],
         req_id: Optional[str],
     ) -> Optional[Negotiator]:
         partner = [_ for _ in partners if _ != self.id][0]
         if not self._obj:
             return None
-        neg = self._obj.create_negotiator(PassThroughSAONegotiator, name=partner, id=partner)
+        neg = self._obj.create_negotiator(
+            ControlledSAONegotiator, name=partner, id=partner
+        )
         return neg
 
     def set_renegotiation_agenda(
@@ -108,7 +108,9 @@ class DefaultOneShotAdapter(Adapter, OneShotUFunCreatorMixin):
     def on_neg_request_rejected(self, req_id: str, by: Optional[List[str]]):
         pass
 
-    def on_neg_request_accepted(self, req_id: str, mechanism: AgentMechanismInterface):
+    def on_neg_request_accepted(
+        self, req_id: str, mechanism: NegotiatorMechanismInterface
+    ):
         pass
 
 
@@ -134,7 +136,7 @@ class _SystemAgent(DefaultOneShotAdapter):
         initiator: str,
         issues: List[Issue],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
     ) -> Optional[Negotiator]:
         pass
 
@@ -151,13 +153,13 @@ class _SystemAgent(DefaultOneShotAdapter):
         self,
         partners: List[str],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
         state: MechanismState,
     ) -> None:
         pass
 
     def on_negotiation_success(
-        self, contract: Contract, mechanism: AgentMechanismInterface
+        self, contract: Contract, mechanism: NegotiatorMechanismInterface
     ) -> None:
         pass
 
