@@ -1,15 +1,30 @@
+from __future__ import annotations
+
 from collections import namedtuple
-from copy import deepcopy
-import warnings
-from typing import Iterable, Tuple, Union, List, Collection, Optional
+from copy import copy
+from functools import lru_cache
+from typing import Iterable
+from typing import List
+from typing import Literal
+from typing import Optional
+from typing import Tuple
+from typing import Union
+from typing import overload
 
 from negmas import Contract
-from negmas.utilities import UtilityFunction, UtilityValue
-from negmas.outcomes import Outcome, Issue
+from negmas.outcomes import Issue
+from negmas.outcomes import Outcome
+from negmas.outcomes import OutcomeSpace
+from negmas.outcomes import make_issue
+from negmas.outcomes import make_os
+from negmas.preferences import StationaryMixin
+from negmas.preferences import UtilityFunction
 
 from scml.scml2020.common import is_system_agent
 
-from .common import QUANTITY, UNIT_PRICE, TIME
+from .common import QUANTITY
+from .common import TIME
+from .common import UNIT_PRICE
 
 __all__ = ["OneShotUFun", "UFunLimit"]
 
@@ -31,7 +46,7 @@ UFunLimit = namedtuple(
 """Information about one utility limit (either highest or lowest). See `OnShotUFun.find_limit` for details."""
 
 
-class OneShotUFun(UtilityFunction):
+class OneShotUFun(StationaryMixin, UtilityFunction):
     """
     Calculates the utility function of a list of contracts or offers.
 
@@ -133,17 +148,47 @@ class OneShotUFun(UtilityFunction):
         self.input_product = input_product
         if self.input_product is not None:
             self.output_product = self.input_product + 1
+        else:
+            self.output_product = None
         if self.normalized:
             self.best = self.find_limit(True, None, None)
             self.worst = self.find_limit(False, None, None)
         else:
             self.best = UFunLimit(*tuple([None] * 10))
             self.worst = UFunLimit(*tuple([None] * 10))
+        if self.input_agent or self.output_agent:
+            # if this is an edge agent, all negotiations will be on the same product so we can define its outcome-space
+            qrange = self.input_qrange if self.output_agent else self.output_qrange
+            prange = self.input_prange if self.output_agent else self.output_prange
+            self.outcome_space = make_os(
+                [
+                    make_issue(qrange, name="quantity"),
+                    make_issue((self.current_step, self.current_step), name="time"),
+                    make_issue(prange, name="unit_price"),
+                ]
+            )
+        else:
+            # if this is not an edge agent, we have a different outcome space for each side
+            self.outcome_spaces = [
+                make_os(
+                    [
+                        make_issue(qrange, name="quantity"),
+                        make_issue((self.current_step, self.current_step), name="time"),
+                        make_issue(prange, name="unit_price"),
+                    ]
+                )
+                for qrange, prange in (
+                    (self.input_qrange, self.input_prange),
+                    (self.output_qrange, self.output_qrange),
+                )
+            ]
+        # slightly bias toward agreements
+        self.reserved_value = self.from_contracts([], ignore_exogenous=False) - 1e-3
 
     def xml(self, issues) -> str:
         raise NotImplementedError("Cannot convert the ufun to xml")
 
-    def __call__(self, offer) -> float:
+    def eval(self, offer) -> float:
         """
         Calculates the utility function given a single contract.
 
@@ -153,7 +198,7 @@ class OneShotUFun(UtilityFunction):
         """
         if not self.input_agent and not self.output_agent:
             return float("-inf")
-        return self.from_offers([offer], [self.input_agent])
+        return self.from_offers((tuple(offer) if offer else None,), (self.input_agent,))
 
     def from_contracts(
         self, contracts: Iterable[Contract], ignore_exogenous=True
@@ -186,7 +231,7 @@ class OneShotUFun(UtilityFunction):
             is_output = product == output_product
             outputs.append(is_output)
             offers.append(self.outcome_as_tuple(c.agreement))
-        return self.from_offers(offers, outputs)
+        return self.from_offers(tuple(offers), tuple(outputs))
 
     @staticmethod
     def outcome_as_tuple(offer):
@@ -198,9 +243,31 @@ class OneShotUFun(UtilityFunction):
             return tuple(outcome)
         return tuple(offer)
 
+    @overload
     def from_offers(
-        self, offers: Iterable[Tuple], outputs: Iterable[bool], return_producible=False
-    ) -> Union[float, Tuple[float, int]]:
+        self,
+        offers: tuple[tuple[int, int, int], ...],
+        outputs: tuple[bool],
+        return_producible: Literal[False] = False,
+    ) -> float:
+        ...
+
+    @overload
+    def from_offers(
+        self,
+        offers: tuple[tuple[int, int, int], ...],
+        outputs: tuple[bool],
+        return_producible: Literal[True],
+    ) -> tuple[float, float]:
+        ...
+
+    @lru_cache
+    def from_offers(
+        self,
+        offers: tuple[tuple[int, int, int], ...],
+        outputs: tuple[bool],
+        return_producible=False,
+    ) -> float | tuple[float, int]:
         """
         Calculates the utility value given a list of offers and whether each
         offer is for output or not (= input).
@@ -230,7 +297,7 @@ class OneShotUFun(UtilityFunction):
             return -offer[UNIT_PRICE] if is_output else offer[UNIT_PRICE]
 
         # copy inputs because we are going to modify them.
-        offers, outputs = deepcopy(list(offers)), deepcopy(list(outputs))
+        offers, outputs = list(offers), list(outputs)
         # indicate that all inputs are not exogenous and that we are adding two
         # exogenous contracts after them.
         exogenous = [False] * len(offers) + [True, True]
@@ -248,14 +315,16 @@ class OneShotUFun(UtilityFunction):
         output_offers = []
         # sort contracts in the optimal order of execution: from cheapest when
         # buying and from the most expensive when selling. See `order` above.
-        sorted_offers = list(sorted(zip(offers, outputs, exogenous), key=order))
+        sorted_offers = sorted(zip(offers, outputs, exogenous), key=order)
 
         # we calculate the total quantity we are are required to pay for `qin` and
         # the associated amount of money we are going to pay `pin`. Moreover,
         # we calculate the total quantity we can actually buy given our limited
         # money balance (`qin_bar`).
         for offer, is_output, is_exogenous in sorted_offers:
-            offer = self.outcome_as_tuple(offer)
+            if not offer:
+                continue
+            offer: tuple[int, int, int]
             if is_output:
                 output_offers.append((offer, is_exogenous))
                 continue
@@ -431,18 +500,56 @@ class OneShotUFun(UtilityFunction):
             self.worst = self.find_limit(False)
         return self.worst.utility
 
+    def minmax(self, *args, **kwargs) -> tuple[float, float]:
+        worst, best = self.extreme_outcomes(*args, **kwargs)
+        return self(worst), self(best)
+
+    def extreme_outcomes(
+        self,
+        outcome_space: OutcomeSpace | None = None,
+        issues: Iterable[Issue] | None = None,
+        outcomes: Iterable[Outcome] | None = None,
+        max_cardinality=1000,
+    ) -> tuple[Outcome, Outcome]:
+        product = (
+            self.output_product
+            if self.input_agent
+            else self.input_product
+            if self.output_agent
+            else None
+        )
+        if product is None:
+            raise ValueError(
+                f"Cannot find the utility range of a midlevel agent: {self.id}\n{vars(self)}"
+            )
+        t = self.current_step
+        is_input = int(product == self.input_product)
+        best = self.find_limit(
+            True,
+            n_input_negs=is_input,
+            n_output_negs=1 - is_input,
+        )
+        worst = self.find_limit(
+            False,
+            n_input_negs=is_input,
+            n_output_negs=1 - is_input,
+        )
+        if self.input_agent:
+            worst_outcome = (worst.output_quantity, t, worst.output_price)
+            best_outcome = (best.output_quantity, t, best.output_price)
+        else:
+            worst_outcome = (worst.input_quantity, t, worst.input_price)
+            best_outcome = (best.input_quantity, t, best.input_price)
+        return worst_outcome, best_outcome
+
     def utility_range(
         self,
+        outcome_space: OutcomeSpace | None = None,
         issues: List[Issue] = None,
-        outcomes: Collection[Outcome] = None,
-        infeasible_cutoff: Optional[float] = None,
+        outcomes: List[Outcome] = None,
         return_outcomes=False,
         max_n_outcomes=1000,
-        ami=None,
-    ) -> Union[
-        Tuple[UtilityValue, UtilityValue],
-        Tuple[UtilityValue, UtilityValue, Outcome, Outcome],
-    ]:
+    ) -> Union[Tuple[float, float], Tuple[float, float, Outcome, Outcome],]:
         """
         Finds the utility range and optionally returns the corresponding outcomes
         from a given issue space or in a single negotiation.
@@ -479,53 +586,12 @@ class OneShotUFun(UtilityFunction):
               negotiation not a set of negotiations and under the assumption that
               all other negotiations if any will end in failure
         """
-        if outcomes is not None:
-            warnings.warn(
-                "Using utility_range with outcomes instead of issues is "
-                "extremely inefficient for OneShotUFun"
-            )
-            return super().utility_range(
-                issues, outcomes, infeasible_cutoff, return_outcomes, max_n_outcomes
-            )
-        product = (
-            self.output_product
-            if self.input_agent
-            else self.input_product
-            if self.output_agent
-            else None
-        )
-        if product is None and ami:
-            product = ami.annotation["product"]
-        if product is None and self.ami:
-            product = self.ami.annotation.get("product", None)
-        if product is None:
-            raise ValueError("Cannot find the utility range of a midlevel agent")
-        t = self.current_step
-        is_input = int(product == self.input_product)
-        best = self.find_limit(
-            True,
-            n_input_negs=is_input,
-            n_output_negs=1 - is_input,
-        )
-        worst = self.find_limit(
-            False,
-            n_input_negs=is_input,
-            n_output_negs=1 - is_input,
-        )
         if not return_outcomes:
-            return worst.utility, best.utility
-        if self.input_agent:
-            worst_outcome = (worst.output_quantity, t, worst.output_price)
-            best_outcome = (best.output_quantity, t, best.output_price)
-        else:
-            worst_outcome = (worst.input_quantity, t, worst.input_price)
-            best_outcome = (best.input_quantity, t, best.input_price)
-        return (  # typing: ignore
-            worst.utility,
-            best.utility,
-            worst_outcome,
-            best_outcome,
-        )  # typing: ignore
+            return self.minmax(outcome_space, issues, list(outcomes), max_n_outcomes)  # type: ignore
+        worst, best = self.extreme_outcomes(
+            outcome_space, issues, outcomes, max_n_outcomes
+        )
+        return (self(worst), self(best), worst, best)
 
     def _is_midlevel(self):
         return not self.input_agent and not self.output_agent
@@ -577,11 +643,11 @@ class OneShotUFun(UtilityFunction):
             secured_output_unit_price,
         )
         actual_util = self.from_offers(
-            [
+            (
                 (result.output_quantity, 0, result.output_price),
                 (result.input_quantity, 0, result.input_price),
-            ],
-            [True, False],
+            ),
+            (True, False),
         )
         assert (
             abs(result.utility - actual_util) < 1e-2
@@ -592,9 +658,7 @@ class OneShotUFun(UtilityFunction):
             self.worst = result
         return result
 
-    def register_agrerement(is_output: bool, quantity: int, unit_price: int) -> None:
-        pass
-
+    @lru_cache
     def find_limit_brute_force(
         self,
         best,
@@ -642,8 +706,8 @@ class OneShotUFun(UtilityFunction):
         for i in range(imax):
             for o in range(omax):
                 u, p = self.from_offers(
-                    [(i, 0, ip), (o, 0, op)],
-                    [False, True],
+                    ((i, 0, ip), (o, 0, op)),
+                    (False, True),
                     return_producible=True,
                 )
                 if (best and u >= limit_u) or (not best and u <= limit_u):
@@ -675,11 +739,11 @@ class OneShotUFun(UtilityFunction):
             - This method is **very** optimistic. If it returns `False`, an
               agent should **never** buy at this price. If it returns `True`, it
               may *still be a bad idea* to buy at this price.
-            - If we **buy** at this price, the **best** case scenario is that we pay it 
+            - If we **buy** at this price, the **best** case scenario is that we pay it
               and pay production cost then receive the unit price of one output.
-            - If we do **not** buy at this price, the **worst** case scenario is that 
+            - If we do **not** buy at this price, the **worst** case scenario is that
               we will pay shortfall penalty for one item
-            - We should **NOT** buy if the best case scenario when buying is worse than 
+            - We should **NOT** buy if the best case scenario when buying is worse than
               the worst case scenario when not buying.
             - If called for agents not at the end of the production chain, it will
               always return `True` because in these cases we do not know what the
@@ -693,7 +757,9 @@ class OneShotUFun(UtilityFunction):
         if self.ex_qout < 1:
             return False
         # do not buy at this price if it is **guaranteed** to lead to a loss
-        return (unit_price + self.production_cost - self.ex_pout // self.ex_qout) < self.shortfall_penalty
+        return (
+            unit_price + self.production_cost - self.ex_pout // self.ex_qout
+        ) < self.shortfall_penalty
 
     def ok_to_sell_at(self, unit_price: float) -> bool:
         """
@@ -703,17 +769,17 @@ class OneShotUFun(UtilityFunction):
             - This method is **very** optimistic. If it returns `False`, an
               agent should **never** sell at this price. If it returns `True`, it
               may *still be a bad idea* to sell at this price.
-            - Sales decisions does not affect in any way the amount we pay for 
-              input materials. It only affects the amount we produce, the amout we 
-              get paid in sales and the amount we pay as disposal cost and 
+            - Sales decisions does not affect in any way the amount we pay for
+              input materials. It only affects the amount we produce, the amout we
+              get paid in sales and the amount we pay as disposal cost and
               shortfall penalty.
-            - If we agree to sell an item at this price, the best case scenario 
+            - If we agree to sell an item at this price, the best case scenario
               is that we can actually produce this item and sell it. We pay production
-              cost and receive the given unit price. 
-            - If we do **not** sell at this price, the worst case scenario is that 
+              cost and receive the given unit price.
+            - If we do **not** sell at this price, the worst case scenario is that
               we really needed that sale. In this case, we will pay disposal cost
               for one item.
-            - We should **NOT** sell if the best case scenario when selling is worse than 
+            - We should **NOT** sell if the best case scenario when selling is worse than
               the worst case scenario when not selling.
             - If called for agents not at the beginning of the production chain, it will
               always return `True` because in these cases we do not know what the
