@@ -21,7 +21,6 @@ from typing import Type
 from typing import Union
 
 import networkx as nx
-from networkx.convert import to_dict_of_dicts
 import numpy as np
 import pandas as pd
 from matplotlib.axis import Axis
@@ -34,11 +33,12 @@ from negmas import Issue
 from negmas import Operations
 from negmas import TimeInAgreementMixin
 from negmas import World
+from negmas import make_issue
 from negmas.helpers import get_class
 from negmas.helpers import get_full_type_name
 from negmas.helpers import instantiate
 from negmas.helpers import unique_name
-from negmas.sao import PassThroughSAONegotiator
+from negmas.sao import ControlledSAONegotiator
 from negmas.sao import SAOController
 from negmas.sao import SAONegotiator
 
@@ -53,12 +53,12 @@ from ..scml2020.common import INFINITE_COST
 from ..scml2020.common import SYSTEM_BUYER_ID
 from ..scml2020.common import SYSTEM_SELLER_ID
 from ..scml2020.common import is_system_agent
-
-from .common import OneShotProfile, OneShotExogenousContract, QUANTITY, UNIT_PRICE
-from .sysagents import _SystemAgent, DefaultOneShotAdapter
 from .adapter import OneShotSCML2020Adapter
 from .agent import OneShotAgent
-from .ufun import OneShotUFun
+from .common import OneShotExogenousContract
+from .common import OneShotProfile
+from .sysagents import DefaultOneShotAdapter
+from .sysagents import _SystemAgent
 
 __all__ = ["SCML2020OneShotWorld"]
 
@@ -101,7 +101,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         catalog_prices: np.ndarray,
         profiles: List[OneShotProfile],
         agent_types: List[Type["OneShotAgent"]],
-        agent_params: List[Dict[str, Any]] = None,
+        agent_params: List[Dict[str, Any]],
         catalog_quantities: Union[int, np.ndarray] = 50,
         # breach processing parameters
         financial_report_period=5,
@@ -119,9 +119,11 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         no_logs=False,
         n_steps=1000,
         time_limit=60 * 90,
+        sync_calls=False,
         # mechanism params
         neg_n_steps=20,
-        neg_time_limit=2 * 60,
+        neg_time_limit=None,
+        neg_hidden_time_limit=3 * 60,
         neg_step_time_limit=60,
         negotiation_speed=0,
         avoid_ultimatum=True,
@@ -189,6 +191,11 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
                         avoid_ultimatum=avoid_ultimatum,
                         dynamic_entry=False,
                         max_wait=len(agent_types),
+                        check_offers=True,
+                        enforce_issue_types=True,
+                        cast_offers=True,
+                        hidden_time_limit=neg_hidden_time_limit,
+                        sync_calls=sync_calls,
                     ),
                 )
             },
@@ -217,6 +224,8 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             name=name,
             **kwargs,
         )
+        if not self.bulletin_board:
+            raise ValueError(f"Cannot find the bulletin-board")
         self.bulletin_board.record("settings", 1, "horizon")
         self.bulletin_board.record(
             "settings", publish_trading_prices, "public_trading_prices"
@@ -272,7 +281,9 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             process_outputs=process_outputs,
             catalog_prices=catalog_prices,
             agent_types_final=[get_full_type_name(_) for _ in agent_types],
-            agent_params_final=[copy.deepcopy(_) for _ in agent_params],
+            agent_params_final=[copy.deepcopy(_) for _ in agent_params]
+            if agent_params is not None
+            else agent_params,
             initial_balance_final=initial_balance,
             penalties_scale_final=penalties_scale,
             penalize_bankrupt_for_future_contracts=penalize_bankrupt_for_future_contracts,
@@ -356,7 +367,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             agent_params = []
             for i, name in enumerate(default_names):
                 b = copy.deepcopy(a)
-                b["name"] = name
+                b["name"] = name  # type: ignore
                 agent_params.append(b)
         elif len(agent_params) == 1:
             a = copy.copy(agent_params[0])
@@ -370,7 +381,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
                 for i, (ns, ps) in enumerate(zip(default_names, agent_params)):
                     agent_params[i] = dict(**ps)
                     agent_params[i]["name"] = ns
-        agent_types += [_SystemAgent, _SystemAgent]
+        agent_types += [_SystemAgent, _SystemAgent]  # type: ignore
         agent_params += [
             {"role": SYSTEM_SELLER_ID, "obj": None},
             {"role": SYSTEM_BUYER_ID, "obj": None},
@@ -558,9 +569,9 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             unit_price, time, quantity = self._make_issues(product)
             self._current_issues.append(
                 [
-                    Issue(values(quantity), name="quantity", value_type=int),
-                    Issue(values(time), name="time", value_type=int),
-                    Issue(values(unit_price), name="unit_price", value_type=int),
+                    make_issue(values(quantity), name="quantity"),
+                    make_issue(values(time), name="time"),
+                    make_issue(values(unit_price), name="unit_price"),
                 ]
             )
 
@@ -1166,8 +1177,8 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         """
         current_balance = sum(self._profits[agent.id]) + self.initial_balances[agent.id]
         self.is_bankrupt[agent.id] = (
-            (current_balance < self.bankruptcy_limit) or self.is_bankrupt[agent.id]
-        )
+            current_balance < self.bankruptcy_limit
+        ) or self.is_bankrupt[agent.id]
         report = FinancialReport(
             agent_id=agent.id,
             step=self.current_step,
@@ -1299,7 +1310,9 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             + self._sold_quantity[has_trade, s + 1]
         )
         self._trading_price[has_trade, s + 1] /= self._betas_sum[has_trade, s + 1]
-        self._trading_price[:, s + 1:] = self._trading_price[:, s + 1].reshape((self.n_products, 1))
+        self._trading_price[:, s + 1 :] = self._trading_price[:, s + 1].reshape(
+            (self.n_products, 1)
+        )
         self._traded_quantity += self._sold_quantity[:, s + 1]
         # self._trading_price[has_trade, s] = (
         #         np.sum(self._betas[:s+1] * self._real_price[has_trade, s::-1])
@@ -1379,9 +1392,15 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             "time": self.current_step,
         }
 
+    def _adjust_contract_types(self, contract):
+        for k in ("unit_price", "quantity"):
+            if not isinstance(contract.agreement[k], int):
+                contract.agreement[k] = int(contract.agreement[k] + 0.5)
+        return contract
+
     def on_contract_signed(self, contract: Contract) -> bool:
-        # if any(is_system_agent(_) for _ in contract.partners):
-        #     print(f"Contract with system partner: {str(contract)}")
+        contract = self._adjust_contract_types(contract)
+
         product = contract.annotation["product"]
         bought = contract.agreement["quantity"]
         total_price = bought * contract.agreement["unit_price"]
@@ -1404,9 +1423,11 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         return super().on_contract_signed(contract)
 
     def contract_size(self, contract: Contract) -> float:
+        contract = self._adjust_contract_types(contract)
         return contract.agreement["quantity"] * contract.agreement["unit_price"]
 
     def contract_record(self, contract: Contract) -> Dict[str, Any]:
+        contract = self._adjust_contract_types(contract)
         c = {
             "id": contract.id,
             "seller_name": self.agents[contract.annotation["seller"]].name,
@@ -1690,7 +1711,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             return True
         if negotiators is None:
             negotiators = [
-                controller.create_negotiator(PassThroughSAONegotiator, name=_, id=_)
+                controller.create_negotiator(ControlledSAONegotiator, name=_, id=_)
                 for _ in partners
             ]
         results = [
@@ -1829,7 +1850,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         def values(x: Union[int, Tuple[int, int]]):
             if not isinstance(x, Iterable):
                 return int(x), int(x)
-            return int(x[0]), int(x[1])
+            return int(min(x)), int(max(x))
 
         controllers = dict()
 
@@ -1842,9 +1863,9 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         for product in range(1, self.n_products):
             unit_price, time, quantity = self._make_issues(product)
             self._current_issues[product] = [
-                Issue(values(quantity), name="quantity", value_type=int),
-                Issue(values(time), name="time", value_type=int),
-                Issue(values(unit_price), name="unit_price", value_type=int),
+                make_issue(values(quantity), name="quantity"),
+                make_issue(values(time), name="time"),
+                make_issue(values(unit_price), name="unit_price"),
             ]
             for aid in self.consumers[product]:
                 if is_system_agent(aid) or isinstance(
