@@ -6,8 +6,8 @@ from scml.scml2020.components import MovingRangeNegotiationManager
 from scml.scml2020.components import SupplyDrivenProductionStrategy
 
 from ..components import KeepOnlyGoodPrices
-from ..components.trading import PredictionBasedTradingStrategy
 from ..components.prediction import MarketAwareTradePredictionStrategy
+from ..components.trading import PredictionBasedTradingStrategy
 from ..world import SCML2020Agent
 
 __all__ = ["MovingRangeAgent", "MarketAwareMovingRangeAgent"]
@@ -23,17 +23,10 @@ class MovingRangeAgent(
 
 
 class MarketAwareMovingRangeAgent(MarketAwareTradePredictionStrategy, MovingRangeAgent):
-    def __init__(
-        self,
-        *args,
-        min_price_margin=0.5,
-        max_price_margin=0.5,
-        **kwargs
-    ):
+    def __init__(self, *args, min_price_margin=0.5, max_price_margin=0.5, **kwargs):
         super().__init__(
             *args,
             min_price_margin=min_price_margin,
             max_price_margin=max_price_margin,
             **kwargs
         )
-
