@@ -11,16 +11,17 @@ from typing import Union
 
 import numpy as np
 from negmas import AgentWorldInterface
-from negmas import AspirationMixin
-from negmas import Issue
+from negmas import ControlledNegotiator
 from negmas import LinearUtilityFunction
 from negmas import MechanismState
 from negmas import Outcome
-from negmas import PassThroughNegotiator
+from negmas import PolyAspiration
 from negmas import ResponseType
 from negmas import UtilityFunction
+from negmas import make_issue
+from negmas import make_os
 from negmas import outcome_is_valid
-from negmas.common import AgentMechanismInterface
+from negmas.common import NegotiatorMechanismInterface
 from negmas.events import Notification
 from negmas.events import Notifier
 from negmas.helpers import instantiate
@@ -29,6 +30,7 @@ from negmas.sao import SAONegotiator
 from negmas.sao import SAOResponse
 from negmas.sao import SAOState
 from negmas.sao import SAOSyncController
+from negmas.sao.negotiators.controlled import ControlledSAONegotiator
 
 from scml.scml2020.common import QUANTITY
 from scml.scml2020.common import TIME
@@ -37,7 +39,7 @@ from scml.scml2020.common import UNIT_PRICE
 __all__ = ["StepController", "SyncController"]
 
 
-class StepController(SAOController, AspirationMixin, Notifier):
+class StepController(SAOController, Notifier):
     """A controller for managing a set of negotiations about selling or buying (but not both)  starting/ending at some
     specific time-step.
 
@@ -100,13 +102,20 @@ class StepController(SAOController, AspirationMixin, Notifier):
         negotiator_params = (
             negotiator_params if negotiator_params is not None else dict()
         )
-        self.secured = 0
+        issues = [
+            make_issue((1, max(int(target_quantity), 1)), "quantity"),
+            make_issue((step, step), "time"),
+            make_issue(
+                (int(urange[0]), max(int(urange[0]), int(urange[1]))), "unit_price"
+            ),
+        ]
         if is_seller:
-            self.ufun = LinearUtilityFunction((1, 1, 10))
+            self.ufun = LinearUtilityFunction(weights=(1.0, 1.0, 10.0), issues=issues)
         else:
-            self.ufun = LinearUtilityFunction((1, -1, -10))
+            self.ufun = LinearUtilityFunction(weights=(1.0, -1.0, -10.0), issues=issues)
         negotiator_params["ufun"] = self.ufun
         self.__negotiator = instantiate(negotiator_type, **negotiator_params)
+        self.secured = 0
         self.completed = defaultdict(bool)
         self.step = step
         self.retries: Dict[str, int] = defaultdict(int)
@@ -115,13 +124,16 @@ class StepController(SAOController, AspirationMixin, Notifier):
     def join(
         self,
         negotiator_id: str,
-        ami: AgentMechanismInterface,
+        nmi: NegotiatorMechanismInterface,
         state: MechanismState,
         *,
+        preferences: Optional["Preferences"] = None,
         ufun: Optional["UtilityFunction"] = None,
         role: str = "agent",
     ) -> bool:
-        joined = super().join(negotiator_id, ami, state, ufun=ufun, role=role)
+        joined = super().join(
+            negotiator_id, nmi, state, preferences=preferences, ufun=ufun, role=role
+        )
         if joined:
             self.completed[negotiator_id] = False
         return joined
@@ -129,7 +141,10 @@ class StepController(SAOController, AspirationMixin, Notifier):
     def propose(self, negotiator_id: str, state: MechanismState) -> Optional["Outcome"]:
         if negotiator_id not in self.negotiators.keys():
             return None
-        self.__negotiator._ami = self.negotiators[negotiator_id][0]._ami
+        try:
+            self.__negotiator._nmi = self.negotiators[negotiator_id][0].nmi
+        except:
+            self.__negotiator._nmi = None
         return self.__negotiator.propose(state)
 
     def respond(
@@ -141,7 +156,7 @@ class StepController(SAOController, AspirationMixin, Notifier):
             return ResponseType.END_NEGOTIATION
         # if negotiator_id not in self.negotiators:
         #     breakpoint()
-        self.__negotiator._ami = self.negotiators[negotiator_id][0]._ami
+        self.__negotiator._nmi = self.negotiators[negotiator_id][0].nmi
         return self.__negotiator.respond(offer=offer, state=state)
 
     def __str__(self):
@@ -153,11 +168,11 @@ class StepController(SAOController, AspirationMixin, Notifier):
 
     def create_negotiator(
         self,
-        negotiator_type: Union[str, Type[PassThroughNegotiator]] = None,
+        negotiator_type: Union[str, Type[ControlledNegotiator]] = None,
         name: str = None,
         cntxt: Any = None,
         **kwargs,
-    ) -> PassThroughNegotiator:
+    ) -> ControlledNegotiator:
         neg = super().create_negotiator(negotiator_type, name, cntxt, **kwargs)
         self.completed[neg.id] = False
         return neg
@@ -213,16 +228,23 @@ class StepController(SAOController, AspirationMixin, Notifier):
                     f"{str(self)} negotiating with {partner} on u={self.urange}"
                     f", q=(1,{self.target-self.secured}), u=({tmin}, {tmax})"
                 )
-                self.awi.request_negotiation(
-                    not self.is_seller,
-                    product=self.product,
-                    quantity=(1, self.target - self.secured),
-                    unit_price=self.urange,
-                    time=(tmin, tmax),
-                    partner=partner,
-                    negotiator=neg,
-                    extra=dict(controller_index=self.step, is_seller=self.is_seller),
-                )
+                if (
+                    self.target > self.secured
+                    and self.urange[0] <= self.urange[1]
+                    and tmin <= tmax
+                ):
+                    self.awi.request_negotiation(
+                        not self.is_seller,
+                        product=self.product,
+                        quantity=(1, self.target - self.secured),
+                        unit_price=self.urange,
+                        time=(tmin, tmax),
+                        partner=partner,
+                        negotiator=neg,
+                        extra=dict(
+                            controller_index=self.step, is_seller=self.is_seller
+                        ),
+                    )
 
 
 # our controller
@@ -285,7 +307,7 @@ class SyncController(SAOSyncController):
         return self._price_weight * price + (1 - self._price_weight) * q
 
     def is_valid(self, negotiator_id: str, offer: "Outcome") -> bool:
-        issues = self.negotiators[negotiator_id][0].ami.issues
+        issues = self.negotiators[negotiator_id][0].nmi.issues
         return outcome_is_valid(offer, issues)
 
     def counter_all(
@@ -307,7 +329,7 @@ class SyncController(SAOSyncController):
         utils = np.array(
             [
                 self.utility(
-                    tuple(o), self.negotiators[nid][0].ami.issues[UNIT_PRICE].max_value
+                    tuple(o), self.negotiators[nid][0].nmi.issues[UNIT_PRICE].max_value
                 )
                 for nid, o in offers.items()
             ]
@@ -381,19 +403,20 @@ class SyncController(SAOSyncController):
             The outcome with highest utility and the corresponding utility
         """
         negotiator = self.negotiators[nid][0]
-        if negotiator.ami is None:
+        if negotiator.nmi is None:
             return None, -1000
+        outcomes = negotiator.nmi.discrete_outcomes()
         utils = np.array(
             [
-                self.utility(_, negotiator.ami.issues[UNIT_PRICE].max_value)
-                for _ in negotiator.ami.outcomes
+                self.utility(_, negotiator.nmi.issues[UNIT_PRICE].max_value)
+                for _ in outcomes
             ]
         )
         best_indx = np.argmax(utils)
         self._best_utils[nid] = utils[best_indx]
         if utils[best_indx] < 0:
             return None, utils[best_indx]
-        return negotiator.ami.outcomes[best_indx], utils[best_indx]
+        return outcomes[best_indx], utils[best_indx]
 
     def first_proposals(self) -> Dict[str, "Outcome"]:
         """Gets a set of proposals to use for initializing the negotiation."""
