@@ -1,6 +1,7 @@
 import random
 
 import hypothesis.strategies as st
+import pytest
 from hypothesis import given
 from hypothesis import settings
 from negmas import save_stats
@@ -11,9 +12,10 @@ from scml.scml2020 import BuyCheapSellExpensiveAgent
 from scml.scml2020 import DoNothingAgent
 from scml.scml2020 import RandomAgent
 from scml.scml2020 import SCML2020World
-from scml.scml2020 import is_system_agent, builtin_agent_types
+from scml.scml2020 import builtin_agent_types
+from scml.scml2020 import is_system_agent
 from scml.scml2020.agents.decentralizing import DecentralizingAgent
-from scml.scml2020.world import SCML2021World
+from tests.switches import SCML_RUN2020
 
 random.seed(0)
 
@@ -70,9 +72,42 @@ def generate_world(
     return world
 
 
+@given(buy_missing=st.booleans(), n_processes=st.integers(2, 4))
+@settings(deadline=300_000, max_examples=20)
+@pytest.mark.skipif(
+    condition=not SCML_RUN2020,
+    reason="Environment set to ignore running 2020 tests. See switches.py",
+)
+@pytest.mark.skipif(
+    condition=not SCML_RUN2020,
+    reason="Environment set to ignore running 2020 tests. See switches.py",
+)
+def test_can_run_random_agent(buy_missing, n_processes):
+    agent_type = RandomAgent
+    world = generate_world(
+        [agent_type],
+        buy_missing_products=buy_missing,
+        n_processes=n_processes,
+        name=unique_name(
+            f"scml2020tests/single/{agent_type.__name__}"
+            f"{'Buy' if buy_missing else 'Fine'}{n_processes}",
+            add_time=True,
+            rand_digits=4,
+        ),
+        compact=COMPACT,
+        no_logs=NOLOGS,
+    )
+    world.run()
+    save_stats(world, world.log_folder)
+
+
 @mark.parametrize("agent_type", types)
 @given(buy_missing=st.booleans(), n_processes=st.integers(2, 4))
 @settings(deadline=300_000, max_examples=20)
+@pytest.mark.skipif(
+    condition=not SCML_RUN2020,
+    reason="Environment set to ignore running 2020 tests. See switches.py",
+)
 def test_can_run_with_a_single_agent_type(agent_type, buy_missing, n_processes):
     world = generate_world(
         [agent_type],
@@ -102,6 +137,10 @@ def test_can_run_with_a_single_agent_type(agent_type, buy_missing, n_processes):
     n_processes=st.integers(2, 4),
 )
 @settings(deadline=300_000, max_examples=20)
+@pytest.mark.skipif(
+    condition=not SCML_RUN2020,
+    reason="Environment set to ignore running 2020 tests. See switches.py",
+)
 def test_can_run_with_a_multiple_agent_types(agent_types, buy_missing, n_processes):
     world = generate_world(
         agent_types,
@@ -127,6 +166,10 @@ def test_can_run_with_a_multiple_agent_types(agent_types, buy_missing, n_process
     initial_balance=st.sampled_from([0, 50, 10_000, 10_000_000]),
 )
 @settings(deadline=1500)
+@pytest.mark.skipif(
+    condition=not SCML_RUN2020,
+    reason="Environment set to ignore running 2020 tests. See switches.py",
+)
 def test_nothing_happens_with_do_nothing(buy_missing, n_processes, initial_balance):
     world = generate_world(
         [DoNothingAgent],
@@ -164,7 +207,11 @@ def test_nothing_happens_with_do_nothing(buy_missing, n_processes, initial_balan
 
 
 @given(buy_missing=st.booleans(), n_processes=st.integers(2, 4))
-@settings(deadline=300_000, max_examples=20)
+@settings(deadline=1000_000, max_examples=20)
+@pytest.mark.skipif(
+    condition=not SCML_RUN2020,
+    reason="Environment set to ignore running 2020 tests. See switches.py",
+)
 def test_something_happens_with_random_agents(buy_missing, n_processes):
     world = generate_world(
         [RandomAgent],
@@ -188,6 +235,10 @@ def test_something_happens_with_random_agents(buy_missing, n_processes):
 
 @given(n_processes=st.integers(2, 4))
 @settings(deadline=300_000, max_examples=3)
+@pytest.mark.skipif(
+    condition=not SCML_RUN2020,
+    reason="Environment set to ignore running 2020 tests. See switches.py",
+)
 def test_agents_go_bankrupt(n_processes):
     buy_missing = True
     world = generate_world(
@@ -235,6 +286,10 @@ def test_agents_go_bankrupt(n_processes):
         #     )
 
 
+@pytest.mark.skipif(
+    condition=not SCML_RUN2020,
+    reason="Environment set to ignore running 2020 tests. See switches.py",
+)
 def test_generate():
     world = SCML2020World(
         **SCML2020World.generate(
@@ -245,6 +300,10 @@ def test_generate():
     assert True
 
 
+@pytest.mark.skipif(
+    condition=not SCML_RUN2020,
+    reason="Environment set to ignore running 2020 tests. See switches.py",
+)
 def test_a_tiny_world():
     world = generate_world(
         [DecentralizingAgent],
@@ -259,6 +318,10 @@ def test_a_tiny_world():
     assert True
 
 
+@pytest.mark.skipif(
+    condition=not SCML_RUN2020,
+    reason="Environment set to ignore running 2020 tests. See switches.py",
+)
 def test_graph():
     world = generate_world(
         [DecentralizingAgent],
@@ -278,6 +341,10 @@ def test_graph():
     world.graph((0, world.n_steps), together=True)
 
 
+@pytest.mark.skipif(
+    condition=not SCML_RUN2020,
+    reason="Environment set to ignore running 2020 tests. See switches.py",
+)
 def test_graphs_lead_to_no_unknown_nodes():
     world = SCML2020World(
         **SCML2020World.generate(
