@@ -1,42 +1,41 @@
 import random
 from collections import defaultdict
-import numpy as np
-import pandas as pd
-from numpy.testing import assert_allclose
-from pytest import raises
 
 import hypothesis.strategies as st
+import numpy as np
+import pandas as pd
 import pytest
 from hypothesis import given
 from hypothesis import settings
-from negmas import genius_bridge_is_running, ResponseType
+from negmas import ResponseType
+from negmas import genius_bridge_is_running
 from negmas import save_stats
-from negmas.sao import SAOResponse
-from negmas.genius import Atlas3
-from negmas.genius import GeniusNegotiator
 from negmas.genius import NiceTitForTat
-from negmas.genius import YXAgent
-from negmas.genius.ginfo import ALL_PASSING_NEGOTIATORS
 from negmas.helpers import unique_name
-from negmas.utilities import LinearUtilityAggregationFunction
-from negmas.utilities import LinearUtilityFunction
+from negmas.preferences import LinearAdditiveUtilityFunction
+from negmas.preferences.value_fun import AffineFun
+from negmas.preferences.value_fun import ConstFun
+from negmas.preferences.value_fun import IdentityFun
+from negmas.preferences.value_fun import LinearFun
+from negmas.sao import SAOResponse
+from numpy.testing import assert_allclose
 from pytest import mark
+from pytest import raises
 
 import scml
+from scml.oneshot import OneShotSingleAgreementAgent
 from scml.oneshot import SCML2020OneShotWorld
 from scml.oneshot import builtin_agent_types
-from scml.oneshot.agent import (
-    OneShotIndNegotiatorsAgent,
-    OneShotSyncAgent,
-    OneShotAgent,
-)
-from scml.oneshot.agents import RandomOneShotAgent, GreedySyncAgent
+from scml.oneshot.agent import OneShotAgent
+from scml.oneshot.agent import OneShotIndNegotiatorsAgent
+from scml.oneshot.agent import OneShotSyncAgent
+from scml.oneshot.agents import GreedySyncAgent
+from scml.oneshot.agents import RandomOneShotAgent
 from scml.oneshot.common import QUANTITY
 from scml.oneshot.common import TIME
 from scml.oneshot.common import UNIT_PRICE
 from scml.oneshot.ufun import OneShotUFun
 from scml.scml2020 import is_system_agent
-from scml.scml2020.components import production
 
 random.seed(0)
 
@@ -56,11 +55,6 @@ std_types = scml.scml2020.builtin_agent_types(as_str=False)
 
 class MyOneShotAgent(RandomOneShotAgent):
     def respond(self, negotiator_id, state, offer):
-        if not (
-            negotiator_id in self.awi.my_consumers
-            or negotiator_id in self.awi.my_suppliers
-        ):
-            breakpoint()
         assert (
             negotiator_id in self.awi.my_consumers
             or negotiator_id in self.awi.my_suppliers
@@ -68,11 +62,6 @@ class MyOneShotAgent(RandomOneShotAgent):
         return super().respond(negotiator_id, state, offer)
 
     def propose(self, negotiator_id, state):
-        if not (
-            negotiator_id in self.awi.my_consumers
-            or negotiator_id in self.awi.my_suppliers
-        ):
-            breakpoint()
         assert (
             negotiator_id in self.awi.my_consumers
             or negotiator_id in self.awi.my_suppliers
@@ -166,6 +155,8 @@ def test_quantity_distribution(n_processes):
 @given(n_processes=st.integers(2, 4))
 @settings(deadline=300_000, max_examples=20)
 def test_can_run_with_a_single_agent_type(agent_type, n_processes):
+    if issubclass(agent_type, OneShotSingleAgreementAgent) and n_processes > 2:
+        return
     world = generate_world(
         [agent_type],
         n_processes=n_processes,
@@ -192,6 +183,11 @@ def test_can_run_with_a_single_agent_type(agent_type, n_processes):
 )
 @settings(deadline=300_000, max_examples=20)
 def test_can_run_with_a_multiple_agent_types(agent_types, n_processes):
+    if (
+        any(issubclass(_, OneShotSingleAgreementAgent) for _ in agent_types)
+        and n_processes > 2
+    ):
+        return
     world = generate_world(
         agent_types,
         name=unique_name(
@@ -208,7 +204,7 @@ def test_can_run_with_a_multiple_agent_types(agent_types, n_processes):
     save_stats(world, world.log_folder)
 
 
-@given(n_processes=st.integers(2, 4))
+@given(n_processes=st.integers(2, 2))
 @settings(deadline=300_000, max_examples=20)
 def test_something_happens_with_random_agents(n_processes):
     world = generate_world(
@@ -600,8 +596,8 @@ def _ufun_unit(
 
     assert mx >= mn or mx == mn == 0
     u = ufun.from_offers(
-        [(qin, 0, pin / qin if qin else 0), (qout, 0, pout / qout if qout else 0)],
-        [False, True],
+        ((qin, 0, pin / qin if qin else 0), (qout, 0, pout / qout if qout else 0)),
+        (False, True),
     )
     # u = ufun.from_aggregates(qin, qout, pin, pout)
     # assert mn <= u <= mx, f"{mn}, {u}, {mx}\nworst: {ufun.worst}\nbest: {ufun.best}"
@@ -716,18 +712,18 @@ class MyGeniusIndNeg(OneShotIndNegotiatorsAgent):
         for partner_id in self.awi.my_consumers:
             if self.awi.is_system(partner_id):
                 continue
-            d[partner_id] = LinearUtilityAggregationFunction(
+            d[partner_id] = LinearAdditiveUtilityFunction(
                 dict(
-                    quantity=lambda x: 0.1 * x,
-                    time=lambda x: 0.0,
-                    unit_price=lambda x: x,
+                    quantity=LinearFun(0.1),
+                    time=ConstFun(0),
+                    unit_price=IdentityFun(),
                 ),
                 weights=dict(
                     quantity=0.1,
                     time=0.0,
                     unit_price=0.9,
                 ),
-                outcome_type=tuple,
+                issues=self.awi.current_output_issues,
                 reserved_value=0.0,
             )
         # generate ufuns that prever lower prices when selling
@@ -735,25 +731,25 @@ class MyGeniusIndNeg(OneShotIndNegotiatorsAgent):
             issues = self.awi.current_input_issues
             if self.awi.is_system(partner_id):
                 continue
-            d[partner_id] = LinearUtilityAggregationFunction(
+            d[partner_id] = LinearAdditiveUtilityFunction(
                 dict(
-                    quantity=lambda x: x,
-                    time=lambda x: 0.0,
-                    unit_price=lambda x: issues[UNIT_PRICE].max_value - x,
+                    quantity=IdentityFun(),
+                    time=ConstFun(0.0),
+                    unit_price=AffineFun(-1, issues[UNIT_PRICE].max_value),
                 ),
                 weights=dict(
                     quantity=0.1,
                     time=0.0,
                     unit_price=0.9,
                 ),
-                outcome_type=tuple,
+                issues=self.awi.current_input_issues,
                 reserved_value=0.0,
             )
         return d
 
 
 def test_ind_negotiators():
-    n_processes = 5
+    n_processes = 2
     world = generate_world(
         [MyIndNeg],
         n_processes=n_processes,
@@ -881,7 +877,7 @@ class MySyncAgent(OneShotSyncAgent):
         )
 
     def get_offer(self, negotiator_id: str):
-        ami = self.get_ami(negotiator_id)
+        ami = self.get_nmi(negotiator_id)
         quantity_issue = ami.issues[QUANTITY]
         unit_price_issue = ami.issues[UNIT_PRICE]
 
@@ -999,9 +995,10 @@ class MyRandomAgent(RandomOneShotAgent):
     ],
 )
 def test_trading_prices_updated(n_agents, n_processes, n_steps):
-    from scml.oneshot import SCML2020OneShotWorld
     from negmas.helpers import force_single_thread
 
+    from scml.oneshot import SCML2020OneShotWorld
+
     eps = 1e-3
 
     world = SCML2020OneShotWorld(
@@ -1060,7 +1057,7 @@ class PricePumpingAgent(OneShotAgent):
     """An agent that causes the intermediate price to go up over time"""
 
     def top_outcome(self, negotiator_id):
-        return tuple(_.max_value for _ in self.get_ami(negotiator_id).issues)
+        return tuple(_.max_value for _ in self.get_nmi(negotiator_id).issues)
 
     def propose(self, negotiator_id, state):
         return self.top_outcome(negotiator_id)
@@ -1163,10 +1160,10 @@ def test_price_pumping_bankrupts_random_agents():
 
 # class RationalRandomOneShotAgent(RandomOneShotAgent):
 #     def top_outcome(self, negotiator_id):
-#         return tuple(_.max_value for _ in self.get_ami(negotiator_id).issues)
+#         return tuple(_.max_value for _ in self.get_nmi(negotiator_id).issues)
 #
 #     def bottom_outcome(self, negotiator_id):
-#         return tuple(_.min_value for _ in self.get_ami(negotiator_id).issues)
+#         return tuple(_.min_value for _ in self.get_nmi(negotiator_id).issues)
 #
 #     def propose(self, negotiator_id, state):
 #         outcome = super().propose(negotiator_id, state)
